package io.reisub.openosrs.prayerflick;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Calculations;
import io.reisub.openosrs.util.Util;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.*;
import net.runelite.api.events.AnimationChanged;
import net.runelite.api.events.GameTick;
import net.runelite.api.widgets.WidgetID;
import net.runelite.api.widgets.WidgetInfo;
import net.runelite.client.callback.ClientThread;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.input.KeyListener;
import net.runelite.client.input.KeyManager;
import net.runelite.client.plugins.Plugin;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.api.Prayers;
import net.runelite.client.plugins.iutils.game.Game;
import net.runelite.client.plugins.iutils.game.iWidget;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.awt.event.KeyEvent;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Prayer Flicking",
	description = "Infinite power!",
	enabledByDefault = false
)
@Slf4j
public class Prayerflick extends Plugin implements KeyListener {
	@Inject
	private Game game;

	@Inject
	private Client client;

	@Inject
	private ClientThread clientThread;

	@Inject
	private KeyManager keyManager;

	@Inject
	private Config config;

	@Inject
	private Calculations calc;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	private final int JALTOK_JAD_MAGE_ATTACK = 7592;
	private final int JALTOK_JAD_RANGE_ATTACK = 7593;
	private final int HESPORI_MAGE_ATTACK = 8223;
	private final int HESPORI_RANGE_ATTACK = 8224;

	private ScheduledExecutorService executor;
	private boolean toggleFlicking;
	private boolean firstFlick;
	private boolean toggledOff;
	private volatile QuickPrayer currentOverhead;
	private int level;

	@Override
	protected void startUp() {
		log.info("Starting Chaos Prayer Flicking");

		keyManager.registerKeyListener(this);
		executor = Executors.newSingleThreadScheduledExecutor();
	}

	@Override
	protected void shutDown() {
		log.info("Stopping Chaos Prayer Flicking");

		keyManager.unregisterKeyListener(this);
		executor.shutdownNow();
	}

	@Subscribe
	private void onGameTick(GameTick event) {
		if (client == null || client.getGameState() == null || client.getGameState() != GameState.LOGGED_IN || executor == null) return;

		iWidget quickPrayers = game.widget(WidgetInfo.MINIMAP_QUICK_PRAYER_ORB);
		if (quickPrayers == null) return;

		if (toggleFlicking && quickPrayers.actions()!= null) {
			boolean active = quickPrayers.actions().get(0).equals("Deactivate");

			if (!active && !firstFlick) {
				toggle(calc.random(config.onDelayMin(), config.onDelayMax()), quickPrayers);
				return;
			}

			toggle(calc.random(config.onDelayMin(), config.offDelayMax()), quickPrayers);
			toggle(calc.random(config.offDelayMin(), config.offDelayMax()), quickPrayers);

			if (firstFlick) {
				firstFlick = false;
			}
		} else  if (!toggleFlicking && toggledOff && config.deactivateAfterStopping()) {
			toggledOff = false;
			toggle(calc.random(config.offDelayMin() + 10, config.offDelayMax() + 10), quickPrayers);
		}
	}

	@Subscribe
	private void onAnimationChanged(AnimationChanged event) {
		Actor actor = event.getActor();
		if (actor == null) return;

		if (config.jadPrayerFlick()) {
			switch (actor.getAnimation()) {
				case AnimationID.TZTOK_JAD_MAGIC_ATTACK:
				case JALTOK_JAD_MAGE_ATTACK:
					setPrayer(false, QuickPrayer.PROTECT_FROM_MAGIC);
					game.utils.sendGameMessage("Pray against magic!");
					break;
				case AnimationID.TZTOK_JAD_RANGE_ATTACK:
				case JALTOK_JAD_RANGE_ATTACK:
					setPrayer(false, QuickPrayer.PROTECT_FROM_MISSILES);
					game.utils.sendGameMessage("Pray against missiles!");
					break;
			}
		}

		if (config.hesporiPrayerFlick()) {
			switch (actor.getAnimation()) {
				case HESPORI_MAGE_ATTACK:
					setPrayer(false, QuickPrayer.PROTECT_FROM_MAGIC);
					break;
				case HESPORI_RANGE_ATTACK:
					setPrayer(false, QuickPrayer.PROTECT_FROM_MISSILES);
					break;
			}
		}
	}

	private void toggle(int delay, iWidget widget) {
		executor.schedule(() -> {
			widget.interact(0);
		}, delay, TimeUnit.MILLISECONDS);
	}

	@Override
	public void keyTyped(KeyEvent e) {}

	@Override
	public void keyPressed(KeyEvent e) {
		level = game.baseLevel(Skill.PRAYER);

		if (config.prayerFlickHotkey().matches(e)) {
			if (toggleFlicking) {
				toggledOff = true;
			} else {
				firstFlick = true;
			}

			toggleFlicking = !toggleFlicking;
		} else if (config.hotkeyMelee().matches(e)) {
			setPrayer(config.allowToggleOff(), QuickPrayer.PROTECT_FROM_MELEE);
		} else if (config.hotkeyMissiles().matches(e)) {
			setPrayer(config.allowToggleOff(), QuickPrayer.PROTECT_FROM_MISSILES);
		} else if (config.hotkeyMagic().matches(e)) {
			setPrayer(config.allowToggleOff(), QuickPrayer.PROTECT_FROM_MAGIC);
		} else if (config.hotkeyMeleeBuff().matches(e)) {
			setPrayer(config.allowToggleOff(), QuickPrayer.getBestMeleeBuff(level));
		} else if (config.hotkeyRangedBuff().matches(e)) {
			setPrayer(config.allowToggleOff(), QuickPrayer.getBestRangedBuff(level));
		} else if (config.hotkeyMagicBuff().matches(e)) {
			setPrayer(config.allowToggleOff(), QuickPrayer.getBestMagicBuff(level));
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {}

	private void setPrayer(boolean allowToggleOff, QuickPrayer... quickPrayers) {
		if (!allowToggleOff && quickPrayers[0] == currentOverhead) return;

		if (currentOverhead == quickPrayers[0]) {
			currentOverhead = QuickPrayer.NONE;
		} else if (quickPrayers[0] == QuickPrayer.PROTECT_FROM_MAGIC
				|| quickPrayers[0] == QuickPrayer.PROTECT_FROM_MISSILES
				|| quickPrayers[0] == QuickPrayer.PROTECT_FROM_MELEE) {
			currentOverhead = quickPrayers[0];
		}

		executor.schedule(() -> {
			iWidget quickPrayersWidget = game.widget(WidgetInfo.MINIMAP_QUICK_PRAYER_ORB);
			if (quickPrayersWidget == null) return;

			quickPrayersWidget.interact(1);
			game.waitUntil(() -> {
				iWidget w = game.widget(WidgetID.QUICK_PRAYERS_GROUP_ID, 4);
				return w != null && !w.hidden();
			});

			for (QuickPrayer quickPrayer : quickPrayers) {
				iWidget protection = game.widget(WidgetID.QUICK_PRAYERS_GROUP_ID, 4, quickPrayer.getChildId());
				if (protection == null) return;

				protection.interact(0);
			}

			iWidget update = game.widget(WidgetID.QUICK_PRAYERS_GROUP_ID, 5);
			if (update == null) return;

			update.interact(0);

			if (config.openInventory()) {
				game.openInterface(3);
			}
		}, 0, TimeUnit.MILLISECONDS);
	}
}